from clint.textui import puts, colored
import logging
import pickle, json
import requests
from bs4 import BeautifulSoup as bs

options = ['login', 'status', 'sync', 'info', 'style']

#need edjude for extra data
def handler(mode, args):
    from defined.post_init import COOKIES_PATH
    from .auth import init_session, login
    from .validator import validate_cookies, validate_session
    from .summary import summary
    from .sync import sync
    from .standings import standings
    from .style import style

    flags = args.flags

    if mode == 'login':
        login()
        puts(colored.green("Successfully saved password and logged in"))
        init_session()

    if not validate_cookies():
        init_session()

    # recover previous session
    session = requests.session()
    with open(COOKIES_PATH, 'rb') as cookiesfile:
        session.cookies.update(pickle.load(cookiesfile))

    if not validate_session(session):
        init_session()

    if mode == 'status':
        summary(session, '--solved' in flags or '-s' in flags)
    elif mode == 'sync':
        sync_samples = '--sync-samples' in flags or '-ssa' in flags or '--sync-all' in flags or '-sa' in flags
        sync_statements = '--sync-statements' in flags or '-sst' in flags or '--sync-all' in flags or '-sa' in flags
        sync(session, sync_samples, sync_statements)
    elif mode == 'info':
        standings(session)
    elif mode == 'style':
        style(args)

    exit(0)



import os

#lib path
def FILES_PATH():
    with open(os.path.split(__file__)[0] + "/../files/path.txt", 'r') as folder_constant_file:
        path = folder_constant_file.read()
    return path

#relative to lib paths
CONFIG_PATH = FILES_PATH() + '/config.ini'
COOKIES_PATH = FILES_PATH() + '/cookies.owo'
LINKS_PATH = FILES_PATH() + '/links.json'
ARGS_PATH = FILES_PATH() + '/gcc_args.txt'
DIR_PATH = FILES_PATH() + '/tasks_dir.txt'

def GET_COMPILER():
    with open(ARGS_PATH, "r") as gcc_args:
        return "gcc " + gcc_args.read() + " {}"

def GET_CAOS_FOLDER():
    with open(DIR_PATH, "r") as tasks_dir:
        return tasks_dir.read()



from defined.post_init import ARGS_PATH, GET_COMPILER, GET_CAOS_FOLDER, DIR_PATH
from defined.constants import gcc_string

from clint.textui import puts, colored, prompt

def set_gcc_args():
    puts(colored.yellow("right now compiler is:"))
    puts((f"{GET_COMPILER()[:-2]}"))
    puts(colored.yellow("new line of arguments for compiler (- to keep same, + to set default):"))
    ans = prompt.query('gcc')

    if ans == "-":
        exit(0)

    if ans == "+":
        ans = gcc_string

    #secret option
    if ans == "mac":
        ans = "-O2 -Wall -Werror -Wno-unused-result -std=gnu11 -lm -fsanitize=address -fsanitize=undefined -fno-sanitize-recover=all"

    with open(ARGS_PATH, "w") as gcc_args:
        gcc_args.write(ans)

def set_tasks_dir():
    tasks_dir_path = GET_CAOS_FOLDER()
    if tasks_dir_path == "-":
        puts(colored.red("folder is not chosen!"))
        puts(colored.yellow("folder to save tasks:"))
    else:
        puts(colored.yellow("right now tasks structure will be generated in:"))
        puts((f"{tasks_dir_path}"))
        puts(colored.yellow("new folder to save tasks(- to keep same):"))
    ans = prompt.query(">")

    if ans == "-":
        exit(0)

    with open(DIR_PATH, "w") as tasks_dir:
        tasks_dir.write(ans)



import os

from defined.constants import gcc_string

def init():
    LIB_PATH = os.path.join(os.path.split(__file__)[0], '..')
    FILES_PATH = os.path.join(LIB_PATH, 'files')

    if 'files' not in os.listdir(LIB_PATH):
            os.mkdir(FILES_PATH)

    with open(FILES_PATH + "/path.txt", 'w') as lib_constant_file:
        lib_constant_file.write(FILES_PATH)

    with open(FILES_PATH + "/gcc_args.txt", "w") as gcc_args:
        gcc_args.write(gcc_string)

    with open(FILES_PATH + "/tasks_dir.txt", "w") as tasks_dir:
        tasks_dir.write("-")

    for file_name in ['cookies.owo', 'links.json', 'config.ini']:
        open(FILES_PATH + "/" + file_name, 'a').close()



from defined.post_init import GET_CAOS_FOLDER
from defined.generator import generator_text
from offline.modify import set_tasks_dir
from utils import get_problems, c_file

import os
from bs4 import BeautifulSoup as bs


def sync(session, sync_samples, sync_statements, target_contest='all', extension='.c'):

    tasks_dir_path = GET_CAOS_FOLDER()
    if tasks_dir_path == "-":
        set_tasks_dir()
        tasks_dir_path = GET_CAOS_FOLDER()

    files = os.listdir(tasks_dir_path)
    problems = get_problems(session)

    for problem in problems:
        short_name = problem['short name']
        contest = short_name[:short_name.find('-')]
        if target_contest != 'all' and contest != target_contest:
            continue

        contest_dir = os.path.join(tasks_dir_path, contest)

        if contest not in files:
            os.mkdir(contest_dir)
            files.append(contest)

        num = short_name[short_name.find('-') + 1:]
        task_dir = os.path.join(contest_dir, num)
        if num not in os.listdir(contest_dir) :
            os.mkdir(task_dir)
        c_file_name = c_file(os.listdir(task_dir))
        if not(c_file_name):
            c_file_name = problem["name"].replace("/", "_") + extension
            open(os.path.join(task_dir, c_file_name), 'w').close()
            with open(os.path.join(task_dir, 'generator.py'), 'a') as generator:
                if os.stat(os.path.join(task_dir, 'generator.py')).st_size < 10:
                    generator.write(generator_text)

            if 'tests' not in os.listdir(task_dir) :
                os.mkdir(task_dir + '/tests')

        if sync_statements:
            task_html = session.get(problem['href'])
            soup = bs(task_html.content, 'html.parser')

            start = f'Problem {short_name}: {problem["name"]}'
            finish = '\nSubmit a solution\n\n'
            statement = soup.text[soup.text.find(start) + len(start) + 1: soup.text.find(finish)]

            with open(os.path.join(task_dir, 'statement.txt'), 'w') as statementfile:
                statementfile.write(statement)

        if sync_samples:
            task_html = session.get(problem['href'])
            soup = bs(task_html.content, 'html.parser')

            if 'Input' in soup.text and 'Output' in soup.text:
                sample_input = soup.text[soup.text.find('Input') + 6: soup.text.find('Output')]
                sample_output = (
                    soup.text[soup.text.find('Output') + 7: soup.text.find('\nSubmit a solution\n\n')] + '\n')

                filein_name = os.path.join(task_dir, 'tests', '000.dat')
                open(filein_name, 'a').close()

                fileout_name = os.path.join(task_dir, 'tests', '000.ans')
                open(fileout_name, 'a').close()

                with open(filein_name, 'w') as filein:
                    filein.write(sample_input)
                with open(fileout_name, 'w') as fileout:
                    fileout.write(sample_output)



from defined.post_init import GET_COMPILER, GET_CAOS_FOLDER

from .modify import set_tasks_dir
from utils import c_file
from clint.textui import puts, colored, indent
import os


def test(contest, task):
    #not best solution cause have to add if everywhere and import in a lot of places, but i dont have a better idea withour ruining of order of iimport
    tasks_dir_path = GET_CAOS_FOLDER()
    if tasks_dir_path == "-":
        set_tasks_dir()
        tasks_dir_path = GET_CAOS_FOLDER()
    task_path = os.path.join(tasks_dir_path, contest, task)
    tests_path = os.path.join(task_path, 'tests')

    tests = []
    try:
        tests = os.listdir(tests_path)
    except:
        puts(colored.red(f"Path {tests_path} doesn't exist."))
        exit(0)

    inputs = filter(lambda x: x[x.find('.') + 1:] == 'dat', tests)
    inputs = list(map(lambda x: x[:x.find('.')], inputs))
    if not inputs:
        puts(colored.red(f"No tests found at {tests_path}"))
        exit(0)

    if 'a.out' in os.listdir(task_path):
        os.remove(f'{task_path}/a.out')

    c_file_name = c_file(os.listdir(task_path))
    os.system(GET_COMPILER().format(os.path.join(task_path, c_file_name.replace(" ", "\ ") )))
    if 'a.out' not in os.listdir(tests_path):
        puts(colored.red(f"Compilation error, aborted"))
        exit(0)

    for input in inputs:
        if input + '.ans' not in tests:
            puts(colored.yellow(f"No matching output for test {input}.dat. Skip it."))
            continue

        run_test(contest, task, input, task_path)

    os.remove(f'{task_path}/temp')
    os.remove(f'{task_path}/a.out')


def run_test(contest, task, test, task_path):
    input_path = os.path.join(task_path, 'tests', test + '.dat')
    output_path = os.path.join(task_path, 'tests', test + '.ans')

    os.system(f'{task_path}/a.out < {input_path} > {task_path}/temp'.format())

    with open(output_path, 'r') as output_file:
        with open(f'{task_path}/temp', 'r') as temp_file:
            expected_lines = output_file.readlines()
            resulting_lines = temp_file.readlines()

            if expected_lines == resulting_lines:
                puts(colored.green(f"Test {test}: OK!"))
            else:
                puts(colored.red(f"Test {test}: Failed!"))
                find_diff(expected_lines, resulting_lines)
                exit(0)

def find_diff(expected_lines, resulting_lines):
    count = 0
    for (line, (expected, resulting)) in enumerate(zip(expected_lines, resulting_lines)):
        if expected != resulting:
            with indent(quote=f'  Line #{line + 1} > '):
                puts(colored.blue(f"Expected:  {expected}"), newline=False)
                puts(colored.yellow(f"Resulting: {resulting}"))
            count += 1

        if count == 10:
            break

    puts(colored.red(f"Resulting output saved to the 'temp' file."))



from defined.post_init import CONFIG_PATH, GET_CAOS_FOLDER
from defined.constants import CLANG_FORMAT_STYLE_STRING
from offline.modify import set_tasks_dir

import configparser
import os
import shutil
from clint.textui import puts, colored, indent


def style(args):
    tasks_dir_path = GET_CAOS_FOLDER()
    if tasks_dir_path == "-":
        set_tasks_dir()
        tasks_dir_path = GET_CAOS_FOLDER()
    config = configparser.ConfigParser()
    config['Tools'] = {}
    config.read(CONFIG_PATH)
    #no idea how this works
    if '--set-tool' in args.grouped:
        config['Tools']['clang-format'] = args.grouped['--set-tool']
        with open(CONFIG_PATH, 'w') as configfile:
            config.write(configfile)

        puts(colored.green('clang-format tool path changed succesfully!'))
        exit(0)

    tool = args.grouped.get(
        '-t', config['Tools'].get('clang-format',
                                  shutil.which('clang-format')))

    if not tool or not os.path.exists(tool):
        puts(
            colored.red(
                "clang-format is not found in $PATH and not specified explicitly."
                "Try modifying $PATH, or use one of '--set-tool' and '-t' flags."
            ))
        exit(1)

    files = []
    if '--all' in args.flags:
        for contest in os.listdir(tasks_dir_path):
            files += [
                os.path.join(tasks_dir_path, contest, task, 'main.c')
                for task in os.listdir(os.path.join(tasks_dir_path, contest))
            ]
    elif args.files:
        files = args.files

    command = (f"{tool}"
               f"-style={CLANG_FORMAT_STYLE_STRING}"
               "--verbose -i"
               f"{' '.join(str(file) for file in files)}")
    os.system(command)



from clint.textui import puts, colored

def print_help():
        puts(colored.yellow("general commands"))
        puts("init\t: reboot the settings of project and setup everything")
        puts("help\t: this info")
        puts("args\t: change args for gcc compiler")
        puts("folder\t: change folder with tasks")

        puts(colored.yellow("ejudge commands"))
        puts("login\t: saves login and password for caos.ejudge.ru")
        puts("info\t: outputs standings info")
        puts("status\t: your tasks status after you send them")

        puts(colored.yellow("tasks commands"))
        puts("test\t: tests program on your local tests")
        puts("\t\t -c contest name \t\t *required")
        puts("\t\t -t task number \t\t *required")
        puts("style\t: changes your local tasks style")
        puts("\t\t -all idk why just add it \t *required")
        puts("sync\t: creates folders for your tasks in a local folder")
        puts("\t\t -ssa, --sync-samples also download first test \t\t *optional")
        puts("\t\t -sst, --sync-statements also download statements\t *optional")
        puts("\t\t -sa, --sync-all combination of previous options\t *optional")



from defined.post_init import COOKIES_PATH
from defined.constants import SUMMARY
from utils import read_links

from clint.textui import puts, colored
import json
import os

def validate_cookies():
    #just checking if not empty
    if os.stat(COOKIES_PATH).st_size < 20:
        return 0
    return 1

def validate_session(session):
    links = read_links()

    result = session.get(links[SUMMARY])
    if 'Invalid session' in result.text:
        return 0
    return 1



# links
SETTINGS = 'Settings'
SUMMARY = 'Summary'
SUBMISSIONS = 'Submissions'
STANDINGS = 'User standings'
CLAR = 'Submit clar'
CLARS = 'Clars'

# submission status
OK = 'OK'
REVIEW = 'Pending review'
NOT_SUBMITTED = 'Not submitted'

gcc_string = "-O2 -Wall -Werror -Wno-unused-result -std=gnu11 -lm -fsanitize=address -fsanitize=undefined -fno-sanitize-recover -fsanitize=leak"

CLANG_FORMAT_STYLE_STRING = """\"{
    Language: Cpp,
    BasedOnStyle: Google,
    IndentWidth: 4,
    UseTab: Never,
    NamespaceIndentation: All,
    ColumnLimit: 80,
    AccessModifierOffset: -4,
    AlignAfterOpenBracket: AlwaysBreak,
    AlignOperands: false,
    AlwaysBreakTemplateDeclarations: Yes,
    BinPackArguments: false,
    BinPackParameters: false,
    AllowShortFunctionsOnASingleLine: Empty,
    BreakBeforeBraces: Custom,
    BraceWrapping: { AfterEnum: true, AfterStruct: true }
}\""""



from defined.constants import STANDINGS
from utils import read_links

from bs4 import BeautifulSoup as bs
from clint.textui import puts, colored


def standings(session):
    links = read_links()
    standings_html = session.get(links[STANDINGS])

    soup = bs(standings_html.content, 'html.parser')

    name = soup.title.text[:soup.title.text.find('[') - 1]

    users = soup.find_all('td', {'class': 'st_team'})
    totals = soup.find_all('td', {'class': 'st_total'})
    scores = soup.find_all('td', {'class': 'st_score'})

    for place, (user, total, score) in enumerate(zip(users, totals, scores)):
        row = "{: >5}.{: >25}: {}({})".format(place + 1, user.text, total.text, score.text)
        if user.text != name:
            puts(colored.blue(row))
        else:
            puts(colored.red(row))



from defined.constants import SUMMARY, OK, REVIEW, NOT_SUBMITTED
from utils import get_problems

from bs4 import BeautifulSoup as bs
from clint.textui import puts, colored, indent

def summary(session, show_solved):
    problems = get_problems(session)

    problem_template = "{: <32}{}({})"
    problems_solved = 0
    total_score = 0
    for problem in problems:
        formatted_problem = problem_template.format(
            problem['name'] + ":",
            problem['status'],
            problem['score']
        )

        total_score += int(problem['score'])
        problems_solved += problem['status'] in [OK, REVIEW]

        if problem['status'] == OK and show_solved:
            with indent(4, quote='✔'):
                puts(colored.green(formatted_problem))
        elif problem['status'] == REVIEW and show_solved:
            with indent(4, quote='?'):
                puts(colored.yellow(formatted_problem))
        elif problem['status'] not in [OK, REVIEW]:
            with indent(4, quote='✖'):
                puts(colored.red(formatted_problem))

    puts('{:_>50}'.format(''))
    puts(colored.green(
        f'Total tasks: {len(problems)}, Solved tasks: {problems_solved}, Total score: {total_score}'
    ))



from defined.constants import SUMMARY, NOT_SUBMITTED
from defined.post_init import LINKS_PATH

from bs4 import BeautifulSoup as bs
import json


def read_links():
    links = {}
    with open(LINKS_PATH, 'r') as linksfile:
        links = json.load(linksfile)
    return links


def get_problem_containers(session):
    links = read_links()
    summary_page = session.get(links[SUMMARY])

    soup = bs(summary_page.content, 'html.parser')
    problem_containers = soup.find_all('td', {'class': 'b1'})

    return problem_containers


def get_problems(session):
    problem_containers = get_problem_containers(session)

    problem = 0
    problems = []
    while problem < len(problem_containers):
        short_name = problem_containers[problem].text
        name = problem_containers[problem + 1].text
        href = problem_containers[problem + 1].a['href']
        status = NOT_SUBMITTED if problem_containers[problem + 2].text == '\xa0' else problem_containers[problem + 2].text
        tests_passed = 0 if problem_containers[problem + 3].text == '\xa0' else problem_containers[problem + 3].text
        score = 0 if problem_containers[problem + 4].text == '\xa0' else problem_containers[problem + 4].text

        problems.append({
            'short name': short_name,
            'name': name,
            'href': href,
            'status': status,
            'tests passed': tests_passed,
            'score': score,
        })

        problem += 6

    return problems

def c_file(folder):
    for file_name in folder:
        if file_name[-2:] == ".c":
            return file_name
    return 0



from clint import arguments
from clint.textui import puts, colored

#i am thinking of removing puts/prompt.query for print/input
#this is far from ideal but a little more abstraction
if __name__ == '__main__':
    mode = arguments.Args()[0]
    args = arguments.Args()

    #project init
    import initialization
    if mode in initialization.options:
        initialization.handler(mode, args)

    #edjudje is needed
    import src
    if mode in src.options:
        src.handler(mode, args)

    #just working with material on computer
    import offline
    if mode in offline.options:
        offline.handler(mode, args)

    puts(colored.red(f"No valid arguments were found"))



from setuptools import setup, find_packages

setup(name = 'caos-reborn',
    version = '1.8.2',
    url = 'https://github.com/DoomzD/caos-reborn',
    author = 'Artem Melnikov & Alex Koryakov',
    author_email = 'melnikovam10@gmail.com',
    description = 'Multifunctional tool for caos course',
    packages = find_packages(),
    scripts = ['caos'],
    install_requires = ['clint', 'beautifulsoup4'])



from clint.textui import puts, colored

options = ['test', 'args', 'folder', 'help']

def handler(mode, args):
    from .test import test
    from .modify import set_gcc_args, set_tasks_dir
    from .help import print_help

    if mode == 'test':
        grouped = args.grouped
        if '-c' not in grouped or '-t' not in grouped or len(grouped['-c']) == 0 or len(grouped['-t']) == 0:
            puts(colored.red("Provide both -c and -t flags for testing."))
            exit(0)
        test(grouped['-c'][0], grouped['-t'][0])
    elif mode == 'args':
        set_gcc_args()
    elif mode == 'folder':
        set_tasks_dir()
    elif mode == 'help':
        print_help()

    exit(0)



from defined.post_init import CONFIG_PATH, COOKIES_PATH, LINKS_PATH

import os
import requests
import configparser
import json, pickle
from bs4 import BeautifulSoup as bs
from clint.textui import puts, colored, prompt
from getpass import getpass

def login():

    login = prompt.query('Tell me your login: ')
    password = getpass(prompt='Password: ', stream=None)
    group = prompt.query('Tell me you group number from the caos.ejudge.ru: ')

    config = configparser.ConfigParser()
    config['Credentials'] = {}
    config['Group'] = {}

    config['Credentials']['login'] = login
    config['Credentials']['password'] = password
    config['Group']['group_number'] = group

    with open(CONFIG_PATH, 'w') as configfile:
        config.write(configfile)


def init_session():
    config = configparser.ConfigParser()
    config.read(CONFIG_PATH)

    if (not 'Credentials' in config) or (not 'Group' in config):
        login()
        config.read(CONFIG_PATH)

    credentials = {
        'login': config['Credentials']['login'],
        'password': config['Credentials']['password']
    }

    session = requests.session()

    url = f'https://caos.ejudge.ru/ej/client?contest_id=1{int(config["Group"]["group_number"])}'
    start_page = session.post(
        url,
        data=credentials,
        headers=dict(referer=url),
    )

    soup = bs(start_page.content, 'html.parser')
    if 'Permission denied' in soup.text:
        puts(colored.red("Invalid login or password. Run 'caos login' again with correct credentials"))
        exit(0)

    links = soup.find_all('a', {'class': 'menu'})[:-1]

    with open(LINKS_PATH, 'w') as linksfile:
        linksfile.write(json.dumps(dict(map(lambda x: (x.text, x['href']), links)), indent=2))

    with open(COOKIES_PATH, 'wb') as cookiesfile:
        pickle.dump(session.cookies, cookiesfile)
